{
  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs/nixpkgs-unstable";
    flake-utils.url = "github:numtide/flake-utils";
  };

  outputs = { self, nixpkgs, flake-utils }:
    # Env #
    let
      env = { };
      exportVars = e: nixpkgs.lib.concatStringsSep "\n" (nixpkgs.lib.mapAttrsToList (name: value: "export ${name}=\"${value}\"") e);
      exportedAllAEvnVars = exportVars env;
    in
    # Helpers #
    let
      mkPgShell =
        { pkgs
          # TODO: Lets not string use pkgs instead
        , postgresqlVersion ? "postgresql_15"
        , postgresqlPort ? 5432
        , extraPostgresqlPackages ? [ ]
        , initialScript ? "SELECT 1;"
        , hbaConf ? ''
            host all all 0.0.0.0/0 trust
            local all all trust
          ''
        }:
        let
          postgresql = pkgs.${postgresqlVersion}.withPackages (ps: builtins.map (name: ps.${name}) extraPostgresqlPackages);

          # postgresqlConf = pkgs.writeText "postgresql.conf" ''
          #   log_duration = off
          #   log_statement = 'ddl'
          #   log_connections = off
          #   log_disconnections = off
          #   log_destination = 'stderr'
          #   listen_addresses = '*'
          # '';

          postgresqlConf = pkgs.writeText "postgresql.conf" ''
            # Logging settings (as before)
            log_duration = off
            log_statement = 'ddl'
            log_connections = off
            log_disconnections = off
            log_destination = 'stderr'
            listen_addresses = '*'

            # Connection limits: lower this if you don't need many connections.
            max_connections = 100

            # Memory configuration:
            # Allocate around 25% of your RAM to shared_buffers.
            shared_buffers = 4GB
            # Estimate effective_cache_size to about 75% of your memory.
            effective_cache_size = 12GB
            # Increase work_mem for sorting and hashing per query.
            work_mem = 16MB
            # Increase maintenance_work_mem for maintenance operations.
            maintenance_work_mem = 1GB

            # Checkpoint settings: longer intervals and smoother checkpoints reduce I/O spikes.
            checkpoint_timeout = 15min
            checkpoint_completion_target = 0.9
            wal_buffers = 16MB

            # Development-only settings: disable durability for faster performance.
            fsync = off
            synchronous_commit = off
          '';

          initialScriptFile = pkgs.writeText "initial.sql" initialScript;

          postgresqlServiceStart = pkgs.writeShellScriptBin "service-start" ''
            set -euo pipefail

            TEMP_DIR="$(mktemp --directory)"
            mkdir -p "$TEMP_DIR/data"

            echo "TEMP_DIR: $TEMP_DIR"

            cleanup() {
              pg_ctl stop -w --mode immediate || true
              rm -rf "$TEMP_DIR"
            }

            trap cleanup EXIT

            export PGDATA=$TEMP_DIR/data
            export PGUSER="postgres"
            export PGPORT="${toString postgresqlPort}"

            pg_ctl initdb -o "-U $PGUSER" -o "--locale=en_US.UTF-8" -o "--encoding=UTF8" --silent -w
            cat "${postgresqlConf}" > $PGDATA/postgresql.conf
            echo -n "${toString hbaConf}" > $PGDATA/pg_hba.conf
            pg_ctl start --silent -w
            # psql -d postgres -U postgres -v ON_ERROR_STOP=1 --file ./init.sql
            # psql -d postgres -U postgres -v ON_ERROR_STOP=1 --file ./fulltext_search.sql
            pgcli --less-chatty --row-limit 10 -d postgres -U postgres
          '';
        in
        pkgs.mkShell {
          buildInputs = [
            pkgs.pgcli
            pkgs.pgtap
            pkgs.perl538Packages.TAPParserSourceHandlerpgTAP
            postgresql
            postgresqlServiceStart
          ];
          shellHook = ''
            clear
            service-start
            exit 0
          '';
        };
    in
    flake-utils.lib.eachDefaultSystem
      (system:
        let
          pkgs = nixpkgs.legacyPackages.${system};
        in
        {
          devShells = {
            pg = mkPgShell {
              inherit pkgs;
              postgresqlVersion = "postgresql_17";
              postgresqlPort = 49264;
              extraPostgresqlPackages = [ "postgis" "pg_uuidv7" ];
            };
            default = pkgs.mkShell rec {
              nativeBuildInputs = with pkgs; [ pkg-config gdal geos ];
              buildInputs = with pkgs; [
                cargo
                rustc
                postgrest
                postgresql_15
                pg_tileserv
                tippecanoe
                samply
              ];
            };
          };
          formatter = pkgs.nixpkgs-fmt;
        }
      );
}
