
CREATE domain "application/x-protobuf" AS bytea;

CREATE OR REPLACE FUNCTION get_mvt_tile(z integer, x integer, y integer)
RETURNS "application/x-protobuf" AS $$
DECLARE
  tile_box geometry;
BEGIN
  -- Compute the tile envelope in EPSG:3857.
  tile_box := ST_TileEnvelope(z, x, y);

  RETURN (
    SELECT ST_AsMVT(tile, 'parcels', 4096, 'geometry')
    FROM (
      SELECT p.*,
         ST_AsMVTGeom(
           ST_Transform(p.geometry, 3857), -- Transform from 4326 to 3857 if needed
           tile_box,
           4096,  -- extent: size of the tile in MVT coordinate space
           256,   -- buffer size (optional, can be adjusted)
           true   -- clip geometries to tile boundaries
         ) AS geom
      FROM parcels p
      WHERE ST_Intersects(
        ST_Transform(p.geometry, 3857),
        tile_box
      )
    ) AS tile
  );
END;
$$ LANGUAGE plpgsql STABLE;

CREATE OR REPLACE FUNCTION get_geojson_tile(z integer, x integer, y integer)
RETURNS json AS $$
DECLARE
  tile_box geometry;
BEGIN
  -- Compute the tile envelope in EPSG:3857 and transform it to EPSG:4326.
  tile_box := ST_Transform(ST_TileEnvelope(z, x, y), 4326);

  RETURN (
    SELECT json_build_object(
      'type', 'FeatureCollection',
      'features', COALESCE(json_agg(
        json_build_object(
          'type', 'Feature',
          'geometry', ST_AsGeoJSON(ST_Intersection(p.geometry, tile_box))::json,
          'properties', to_jsonb(p) - 'geometry'
        )
      ), '[]'::json)
    )
    FROM parcels p
    WHERE ST_Intersects(p.geometry, tile_box)
  );
END;
$$ LANGUAGE plpgsql STABLE;
